/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

package com.android.dx.rop.type;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class TypeTest {

    @Test
    public void testClearInternTable() {
        Type boolArray = Type.BOOLEAN.getArrayType();
        assertTrue(boolArray == Type.BOOLEAN_ARRAY);
        assertTrue(boolArray == Type.intern("[Z"));
        Type myClass = Type.intern("Lcom/example/Foo;");

        Type.clearInternTable();

        assertTrue(boolArray == Type.intern("[Z"));
        Type myClass2 = Type.intern("Lcom/example/Foo;");
        assertEquals(myClass, myClass2);
        assertFalse(myClass == myClass2);
    }
}