/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import constmethodhandle.ConstTest;
import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;

public class Main{
  public static void main(String[] args) throws Throwable {
    // At compilation time, ConstTest.main has not yet been
    // generated. It is generated before execution / linking.
    MethodHandle generatedMain = MethodHandles.lookup()
        .findStatic(ConstTest.class, "main", MethodType.methodType(void.class, String[].class));
    generatedMain.invokeExact(args);
  }
}
